// ZEVA LVC12: Low Voltage Cutoff for 12V batteries
// Open Source version, released under MIT License (see Readme file)
// Last modified by Ian Hooper, August 2021

// Code for ATtiny13A
// Fuses: All defaults (no changes)

#include <avr/io.h>
#include <util/delay.h>
#include <avr/wdt.h>

// Voltage sampler uses 100:47 divider with ~5.05V reference -> 65 ADCs per volt
// Usual cutoff is 11.5V, *65 = 748 ADCs
#define THRESHOLD 748
#define DIODE_OFFSET	58 // Account for ~0.9V drop across BAV3004W diode

// Analog to Digital sampling function
#define ADC_VREF_TYPE 0x00 //0x40 // this woulda turned on Vref = 1.1V
unsigned short ReadADC(unsigned char adc_input)
{
	ADMUX = adc_input | (ADC_VREF_TYPE & 0xff);
	_delay_us(10); // Delay needed for the stabilization of the ADC input voltage
	ADCSRA |= 0x40; // Start the AD conversion
	while ((ADCSRA & 0x10) == 0); // Wait for the AD conversion to complete
	ADCSRA |= 0x10;
	return ADCW;
}

int main(void)
{
	_delay_ms(100); // Little delay to let things settle

	wdt_enable(WDTO_15MS); // Enable watchdog timer
	DDRB = 0b00000110; // PB1 was output on older version, PB2 on newer - just use both for backwards compatibility
	ADCSRA=0x84; // ADC initialization

	PORTB = 0b00000110; // Enable output(s)

	int counter = 0;
	while (1)
	{
		wdt_reset(); // Heartbeat for watchdog timer
		
		if (ReadADC(2) < THRESHOLD-DIODE_OFFSET) // Then we are undervoltage
		{
			if (counter < 10000) counter++;
		}
		else
		{
			if (counter > 120) counter = 120; // Reset to low value instantly (for faster re-enable really)
			if (counter > 0) counter--; // Keep counting down for 1 sec reset
		}

		// Counter of 1200 represents 10 seconds (~120hz main loop), enough time to start vehicle after reset
		// (And avoids tripping due to voltage sag from transient loads)
		if (counter > 1200) PORTB = 0; // Disable output
		if (counter == 0) PORTB = 0b00000110; // Re-enable output if voltage has recovered for over 1 second

		_delay_ms(10); // Cycle main loop at about 120Hz (clock is 1.2Mhz but delay function is using 1Mhz)
	}
    
    return 0; // Never gets here but compiler wants to see it
}


